#!/usr/bin/env python
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Search for branches underneath a directory and serve them all."""

import logging
import os
import sys

from optparse import OptionParser

from paste import httpserver
from paste.httpexceptions import HTTPExceptionHandler
from paste.translogger import TransLogger

from loggerhead import __version__
from loggerhead.apps.filesystem import (
    BranchesFromFileSystemRoot, UserBranchesFromFileSystemRoot)

_reloader_environ_key = 'PYTHON_RELOADER_SHOULD_RUN'

def command_line_parser():
    parser = OptionParser("%prog [options] <path>")
    parser.set_defaults(
        user_dirs=False,
        show_version=False,
        )
    parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
                      help="Serve user directories as ~user.")
    parser.add_option("--trunk-dir", metavar="DIR",
                      help="The directory that contains the trunk branches.")
    parser.add_option("--port", dest="user_port",
                      help="Port Loggerhead should listen on (defaults to 8080).")
    parser.add_option("--host", dest="user_host",
                      help="Host Loggerhead should listen on.")
    parser.add_option("--prefix", dest="user_prefix",
                      help="Specify host prefix.")
    parser.add_option("--reload", action="store_true", dest="reload",
                      help="Use a file monitor to restart the application")
    parser.add_option("--version", action="store_true", dest="show_version",
                      help="Print the software version and exit")
    return parser


def main(args):
    logging.basicConfig()
    logging.getLogger().setLevel(logging.DEBUG)

    parser = command_line_parser()
    (options, args) = parser.parse_args(sys.argv[1:])

    if options.show_version:
        print "loggerhead %s" % __version__
        sys.exit(0)

    if len(args) > 1:
        parser.print_help()
        sys.exit(1)
    elif len(args) == 1:
        [path] = args
    else:
        path = '.'

    if not os.path.isdir(path):
        print "%s is not a directory" % path
        sys.exit(1)

    if options.trunk_dir and not options.user_dirs:
        print "--trunk-dir is only valid with --user-dirs"
        sys.exit(1)
        
    if options.reload:
        if os.environ.get(_reloader_environ_key):
            from paste import reloader
            reloader.install(int(1))
        else:
            return restart_with_reloader()

    if options.user_dirs:
        if not options.trunk_dir:
            print "You didn't specify a directory for the trunk directories."
            sys.exit(1)
        app = UserBranchesFromFileSystemRoot(path, options.trunk_dir)
    else:
        app = BranchesFromFileSystemRoot(path)
    app = HTTPExceptionHandler(app)
    app = TransLogger(app)

    if not options.user_prefix:
        prefix = '/'
    else:
        prefix = options.user_prefix

    try:
        from paste.deploy.config import PrefixMiddleware
    except ImportError:
        pass
    else:
        app = PrefixMiddleware(app, prefix=prefix)
    
    if not options.user_port:
        port = '8080'
    else:
        port = options.user_port

    if not options.user_host:
        host = '0.0.0.0'
    else:
        host = options.user_host

    httpserver.serve(app, host=host, port=port)
    

def restart_with_reloader():
    print 'Starting subprocess with file monitor'
    while 1:
        args = [sys.executable] + sys.argv
        new_environ = os.environ.copy()
        new_environ[_reloader_environ_key] = 'true'
        proc = None
        try:
            try:
                import paste.script.serve
                import subprocess
                paste.script.serve._turn_sigterm_into_systemexit()
                proc = subprocess.Popen(args, env=new_environ)
                exit_code = proc.wait()
                proc = None
            except KeyboardInterrupt:
                print '^C caught in monitor process'
                return 1
        finally:
            if (proc is not None
                and hasattr(os, 'kill')):
                import signal
                try:
                    os.kill(proc.pid, signal.SIGTERM)
                except (OSError, IOError):
                    pass
            
        # Reloader always exits with code 3; but if we are
        # a monitor, any exit code will restart
        if exit_code != 3:
            return exit_code
        print '-'*20, 'Restarting', '-'*20


if __name__ == "__main__":
    main(sys.argv)
